@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 #
 # A library that provides a Python interface to the Telegram Bot API
-# Copyright (C) 2015-2020
+# Copyright (C) 2015-2021
 # Leandro Toledo de Souza <devs@python-telegram-bot.org>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -16,11 +16,13 @@
 #
 # You should have received a copy of the GNU Lesser Public License
 # along with this program.  If not, see [http://www.gnu.org/licenses/].
+# pylint: disable=R0201
 """This module contains the CallbackContext class."""
 from queue import Queue
-from typing import Dict, Any, Tuple, TYPE_CHECKING, Optional, Match, List, NoReturn, Union
+from typing import TYPE_CHECKING, Dict, List, Match, NoReturn, Optional, Tuple, Union
 
 from telegram import Update
+
 if TYPE_CHECKING:
     from telegram import Bot
     from telegram.ext import Dispatcher, Job, JobQueue
@@ -71,9 +73,8 @@ class CallbackContext:
             is handled by :class:`telegram.ext.CommandHandler`, :class:`telegram.ext.PrefixHandler`
             or :class:`telegram.ext.StringCommandHandler`. It contains a list of the words in the
             text after the command, using any whitespace string as a delimiter.
-        error (:class:`telegram.TelegramError`): Optional. The error that was raised.
-            Only present when passed to a error handler registered with
-            :attr:`telegram.ext.Dispatcher.add_error_handler`.
+        error (:obj:`Exception`): Optional. The error that was raised. Only present when passed
+            to a error handler registered with :attr:`telegram.ext.Dispatcher.add_error_handler`.
         async_args (List[:obj:`object`]): Optional. Positional arguments of the function that
             raised the error. Only present when the raising function was run asynchronously using
             :meth:`telegram.ext.Dispatcher.run_async`.
@@ -91,18 +92,19 @@ class CallbackContext:
             dispatcher (:class:`telegram.ext.Dispatcher`):
         """
         if not dispatcher.use_context:
-            raise ValueError('CallbackContext should not be used with a non context aware '
-                             'dispatcher!')
+            raise ValueError(
+                'CallbackContext should not be used with a non context aware ' 'dispatcher!'
+            )
         self._dispatcher = dispatcher
         self._bot_data = dispatcher.bot_data
-        self._chat_data: Optional[Dict[Any, Any]] = None
-        self._user_data: Optional[Dict[Any, Any]] = None
+        self._chat_data: Optional[Dict[object, object]] = None
+        self._user_data: Optional[Dict[object, object]] = None
         self.args: Optional[List[str]] = None
         self.matches: Optional[List[Match]] = None
         self.error: Optional[Exception] = None
         self.job: Optional['Job'] = None
         self.async_args: Optional[Union[List, Tuple]] = None
-        self.async_kwargs: Optional[Dict[str, Any]] = None
+        self.async_kwargs: Optional[Dict[str, object]] = None
 
     @property
     def dispatcher(self) -> 'Dispatcher':
@@ -114,35 +116,40 @@ class CallbackContext:
         return self._bot_data
 
     @bot_data.setter
-    def bot_data(self, value: Any) -> NoReturn:
-        raise AttributeError("You can not assign a new value to bot_data, see "
-                             "https://git.io/fjxKe")
+    def bot_data(self, value: object) -> NoReturn:
+        raise AttributeError(
+            "You can not assign a new value to bot_data, see " "https://git.io/fjxKe"
+        )
 
     @property
     def chat_data(self) -> Optional[Dict]:
         return self._chat_data
 
     @chat_data.setter
-    def chat_data(self, value: Any) -> NoReturn:
-        raise AttributeError("You can not assign a new value to chat_data, see "
-                             "https://git.io/fjxKe")
+    def chat_data(self, value: object) -> NoReturn:
+        raise AttributeError(
+            "You can not assign a new value to chat_data, see " "https://git.io/fjxKe"
+        )
 
     @property
     def user_data(self) -> Optional[Dict]:
         return self._user_data
 
     @user_data.setter
-    def user_data(self, value: Any) -> NoReturn:
-        raise AttributeError("You can not assign a new value to user_data, see "
-                             "https://git.io/fjxKe")
+    def user_data(self, value: object) -> NoReturn:
+        raise AttributeError(
+            "You can not assign a new value to user_data, see " "https://git.io/fjxKe"
+        )
 
     @classmethod
-    def from_error(cls,
-                   update: object,
-                   error: Exception,
-                   dispatcher: 'Dispatcher',
-                   async_args: Union[List, Tuple] = None,
-                   async_kwargs: Dict[str, Any] = None) -> 'CallbackContext':
+    def from_error(
+        cls,
+        update: object,
+        error: Exception,
+        dispatcher: 'Dispatcher',
+        async_args: Union[List, Tuple] = None,
+        async_kwargs: Dict[str, object] = None,
+    ) -> 'CallbackContext':
         self = cls.from_update(update, dispatcher)
         self.error = error
         self.async_args = async_args
@@ -158,9 +165,9 @@ class CallbackContext:
             user = update.effective_user
 
             if chat:
-                self._chat_data = dispatcher.chat_data[chat.id]
+                self._chat_data = dispatcher.chat_data[chat.id]  # pylint: disable=W0212
             if user:
-                self._user_data = dispatcher.user_data[user.id]
+                self._user_data = dispatcher.user_data[user.id]  # pylint: disable=W0212
         return self
 
     @classmethod
@@ -169,7 +176,7 @@ class CallbackContext:
         self.job = job
         return self
 
-    def update(self, data: Dict[str, Any]) -> None:
+    def update(self, data: Dict[str, object]) -> None:
         self.__dict__.update(data)
 
     @property
